@@ -13,11 +13,15 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#include <stdint.h>
 #include <stdio.h>
 
 #include <cfloat>  // FLT_MAX
 #include <cmath>   // std::abs
 
+#include "hwy/base.h"
+#include "hwy/nanobenchmark.h"
+
 // clang-format off
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "hwy/contrib/math/math_test.cc"
@@ -39,21 +43,15 @@ namespace HWY_NAMESPACE {
 #if HWY_ARCH_X86_32 && HWY_COMPILER_GCC_ACTUAL && \
     (HWY_TARGET == HWY_SCALAR || HWY_TARGET == HWY_EMU128)
 
-// On 32-bit x86 with GCC 13+, build with `-fexcess-precision=standard` - see
+// GCC 13+: because CMAKE_CXX_EXTENSIONS is OFF, we build with -std= and hence
+// also -fexcess-precision=standard, so there is no problem. See #1708 and
 // https://gcc.gnu.org/bugzilla/show_bug.cgi?id=323.
 #if HWY_COMPILER_GCC_ACTUAL >= 1300
-
-#if FLT_EVAL_METHOD == 0  // correct flag given, no problem
 #define HWY_MATH_TEST_EXCESS_PRECISION 0
-#else
-#define HWY_MATH_TEST_EXCESS_PRECISION 1
-#pragma message( \
-    "Skipping scalar math_test on 32-bit x86 GCC 13+ without -fexcess-precision=standard")
-#endif  // FLT_EVAL_METHOD
 
 #else                  // HWY_COMPILER_GCC_ACTUAL < 1300
 
-// On 32-bit x86 with GCC <13, set HWY_CMAKE_SSE2 - see
+// The build system must enable SSE2, e.g. via HWY_CMAKE_SSE2 - see
 // https://stackoverflow.com/questions/20869904/c-handling-of-excess-precision .
 #if defined(__SSE2__)  // correct flag given, no problem
 #define HWY_MATH_TEST_EXCESS_PRECISION 0
@@ -68,14 +66,6 @@ namespace HWY_NAMESPACE {
 #define HWY_MATH_TEST_EXCESS_PRECISION 0
 #endif  // HWY_ARCH_X86_32 etc
 
-template <class Out, class In>
-inline Out BitCast(const In& in) {
-  static_assert(sizeof(Out) == sizeof(In), "");
-  Out out;
-  CopyBytes<sizeof(out)>(&in, &out);
-  return out;
-}
-
 template <class T, class D>
 HWY_NOINLINE void TestMath(const char* name, T (*fx1)(T),
                            Vec<D> (*fxN)(D, VecArg<Vec<D>>), D d, T min, T max,
@@ -87,21 +77,22 @@ HWY_NOINLINE void TestMath(const char* name, T (*fx1)(T),
       fprintf(stderr,
               "Skipping math_test due to GCC issue with excess precision.\n");
     }
+    return;
   }
 
   using UintT = MakeUnsigned<T>;
 
-  const UintT min_bits = BitCast<UintT>(min);
-  const UintT max_bits = BitCast<UintT>(max);
+  const UintT min_bits = BitCastScalar<UintT>(min);
+  const UintT max_bits = BitCastScalar<UintT>(max);
 
   // If min is negative and max is positive, the range needs to be broken into
   // two pieces, [+0, max] and [-0, min], otherwise [min, max].
   int range_count = 1;
   UintT ranges[2][2] = {{min_bits, max_bits}, {0, 0}};
   if ((min < 0.0) && (max > 0.0)) {
-    ranges[0][0] = BitCast<UintT>(static_cast<T>(+0.0));
+    ranges[0][0] = BitCastScalar<UintT>(ConvertScalarTo<T>(+0.0));
     ranges[0][1] = max_bits;
-    ranges[1][0] = BitCast<UintT>(static_cast<T>(-0.0));
+    ranges[1][0] = BitCastScalar<UintT>(ConvertScalarTo<T>(-0.0));
     ranges[1][1] = min_bits;
     range_count = 2;
   }
@@ -116,7 +107,8 @@ HWY_NOINLINE void TestMath(const char* name, T (*fx1)(T),
     for (UintT value_bits = start; value_bits <= stop; value_bits += step) {
       // For reasons unknown, the HWY_MAX is necessary on RVV, otherwise
       // value_bits can be less than start, and thus possibly NaN.
-      const T value = BitCast<T>(HWY_MIN(HWY_MAX(start, value_bits), stop));
+      const T value =
+          BitCastScalar<T>(HWY_MIN(HWY_MAX(start, value_bits), stop));
       const T actual = GetLane(fxN(d, Set(d, value)));
       const T expected = fx1(value);
 
@@ -130,7 +122,7 @@ HWY_NOINLINE void TestMath(const char* name, T (*fx1)(T),
       const auto ulp = hwy::detail::ComputeUlpDelta(actual, expected);
       max_ulp = HWY_MAX(max_ulp, ulp);
       if (ulp > max_error_ulp) {
-        fprintf(stderr, "%s: %s(%f) expected %f actual %f ulp %g max ulp %u\n",
+        fprintf(stderr, "%s: %s(%f) expected %E actual %E ulp %g max ulp %u\n",
                 hwy::TypeName(T(), Lanes(d)).c_str(), name, value, expected,
                 actual, static_cast<double>(ulp),
                 static_cast<uint32_t>(max_error_ulp));
@@ -165,9 +157,11 @@ HWY_NOINLINE void TestMath(const char* name, T (*fx1)(T),
   };                                                                      \
   DEFINE_MATH_TEST_FUNC(NAME)
 
-// Floating point values closest to but less than 1.0
-const float kNearOneF = BitCast<float>(0x3F7FFFFF);
-const double kNearOneD = BitCast<double>(0x3FEFFFFFFFFFFFFFULL);
+// Floating point values closest to but less than 1.0. Avoid variables with
+// static initializers inside HWY_BEFORE_NAMESPACE/HWY_AFTER_NAMESPACE to
+// ensure target-specific code does not leak into startup code.
+float kNearOneF() { return BitCastScalar<float>(0x3F7FFFFF); }
+double kNearOneD() { return BitCastScalar<double>(0x3FEFFFFFFFFFFFFFULL); }
 
 // The discrepancy is unacceptably large for MSYS2 (less accurate libm?), so
 // only increase the error tolerance there.
@@ -190,14 +184,14 @@ constexpr uint64_t ACosh32ULP() {
 template <class D>
 static Vec<D> SinCosSin(const D d, VecArg<Vec<D>> x) {
   Vec<D> s, c;
-  SinCos(d, x, s, c);
+  CallSinCos(d, x, s, c);
   return s;
 }
 
 template <class D>
 static Vec<D> SinCosCos(const D d, VecArg<Vec<D>> x) {
   Vec<D> s, c;
-  SinCos(d, x, s, c);
+  CallSinCos(d, x, s, c);
   return c;
 }
 
@@ -234,15 +228,19 @@ DEFINE_MATH_TEST(Asinh,
 DEFINE_MATH_TEST(Atan,
   std::atan,  CallAtan,  -FLT_MAX,   +FLT_MAX,    3,
   std::atan,  CallAtan,  -DBL_MAX,   +DBL_MAX,    3)
+// NEON has ULP 4 instead of 3
 DEFINE_MATH_TEST(Atanh,
-  std::atanh, CallAtanh, -kNearOneF, +kNearOneF,  4,  // NEON is 4 instead of 3
-  std::atanh, CallAtanh, -kNearOneD, +kNearOneD,  3)
+  std::atanh, CallAtanh, -kNearOneF(), +kNearOneF(),  4,
+  std::atanh, CallAtanh, -kNearOneD(), +kNearOneD(),  3)
 DEFINE_MATH_TEST(Cos,
   std::cos,   CallCos,   -39000.0f,  +39000.0f,   3,
   std::cos,   CallCos,   -39000.0,   +39000.0,    Cos64ULP())
 DEFINE_MATH_TEST(Exp,
   std::exp,   CallExp,   -FLT_MAX,   +104.0f,     1,
   std::exp,   CallExp,   -DBL_MAX,   +104.0,      1)
+DEFINE_MATH_TEST(Exp2,
+  std::exp2,  CallExp2,  -FLT_MAX,   +128.0f,     2,
+  std::exp2,  CallExp2,  -DBL_MAX,   +128.0,      2)
 DEFINE_MATH_TEST(Expm1,
   std::expm1, CallExpm1, -FLT_MAX,   +104.0f,     4,
   std::expm1, CallExpm1, -DBL_MAX,   +104.0,      4)
@@ -285,33 +283,38 @@ void Atan2TestCases(T /*unused*/, D d, size_t& padded,
     T x;
     T expected;
   };
-  const T pos = static_cast<T>(1E5);
-  const T neg = static_cast<T>(-1E7);
-  // T{-0} is not enough to get an actual negative zero.
-  const T n0 = static_cast<T>(-0.0);
+  const T pos = ConvertScalarTo<T>(1E5);
+  const T neg = ConvertScalarTo<T>(-1E7);
+  const T p0 = ConvertScalarTo<T>(0);
+  // -0 is not enough to get an actual negative zero.
+  const T n0 = ConvertScalarTo<T>(-0.0);
+  const T p1 = ConvertScalarTo<T>(1);
+  const T n1 = ConvertScalarTo<T>(-1);
+  const T p2 = ConvertScalarTo<T>(2);
+  const T n2 = ConvertScalarTo<T>(-2);
   const T inf = GetLane(Inf(d));
   const T nan = GetLane(NaN(d));
 
-  const T pi = static_cast<T>(3.141592653589793238);
-  const YX test_cases[] = {                                  // 45 degree steps:
-                           {T{0.0}, T{1.0}, T{0}},           // E
-                           {T{-1.0}, T{1.0}, -pi / 4},       // SE
-                           {T{-1.0}, T{0.0}, -pi / 2},       // S
-                           {T{-1.0}, T{-1.0}, -3 * pi / 4},  // SW
-                           {T{0.0}, T{-1.0}, pi},            // W
-                           {T{1.0}, T{-1.0}, 3 * pi / 4},    // NW
-                           {T{1.0}, T{0.0}, pi / 2},         // N
-                           {T{1.0}, T{1.0}, pi / 4},         // NE
+  const T pi = ConvertScalarTo<T>(3.141592653589793238);
+  const YX test_cases[] = {                        // 45 degree steps:
+                           {p0, p1, p0},           // E
+                           {n1, p1, -pi / 4},      // SE
+                           {n1, p0, -pi / 2},      // S
+                           {n1, n1, -3 * pi / 4},  // SW
+                           {p0, n1, pi},           // W
+                           {p1, n1, 3 * pi / 4},   // NW
+                           {p1, p0, pi / 2},       // N
+                           {p1, p1, pi / 4},       // NE
 
                            // y = ±0, x < 0 or -0
-                           {T{0}, T{-1}, pi},
-                           {n0, T{-2}, -pi},
+                           {p0, n1, pi},
+                           {n0, n2, -pi},
                            // y = ±0, x > 0 or +0
-                           {T{0}, T{2}, T{0}},
-                           {n0, T{2}, n0},
+                           {p0, p2, p0},
+                           {n0, p2, n0},
                            // y = ±∞, x finite
-                           {inf, T{3}, pi / 2},
-                           {-inf, T{3}, -pi / 2},
+                           {inf, p2, pi / 2},
+                           {-inf, p2, -pi / 2},
                            // y = ±∞, x = -∞
                            {inf, -inf, 3 * pi / 4},
                            {-inf, -inf, -3 * pi / 4},
@@ -319,21 +322,21 @@ void Atan2TestCases(T /*unused*/, D d, size_t& padded,
                            {inf, inf, pi / 4},
                            {-inf, inf, -pi / 4},
                            // y < 0, x = ±0
-                           {T{-2}, T{0}, -pi / 2},
-                           {T{-1}, n0, -pi / 2},
+                           {n2, p0, -pi / 2},
+                           {n1, n0, -pi / 2},
                            // y > 0, x = ±0
-                           {pos, T{0}, pi / 2},
-                           {T{4}, n0, pi / 2},
+                           {pos, p0, pi / 2},
+                           {p2, n0, pi / 2},
                            // finite y > 0, x = -∞
                            {pos, -inf, pi},
                            // finite y < 0, x = -∞
                            {neg, -inf, -pi},
                            // finite y > 0, x = +∞
-                           {pos, inf, T{0}},
+                           {pos, inf, p0},
                            // finite y < 0, x = +∞
                            {neg, inf, n0},
                            // y NaN xor x NaN
-                           {nan, T{0}, nan},
+                           {nan, p0, nan},
                            {pos, nan, nan}};
   const size_t kNumTestCases = sizeof(test_cases) / sizeof(test_cases[0]);
   const size_t N = Lanes(d);
@@ -341,7 +344,7 @@ void Atan2TestCases(T /*unused*/, D d, size_t& padded,
   out_y = AllocateAligned<T>(padded);
   out_x = AllocateAligned<T>(padded);
   out_expected = AllocateAligned<T>(padded);
-  HWY_ASSERT(out_y && out_x);
+  HWY_ASSERT(out_y && out_x && out_expected);
   size_t i = 0;
   for (; i < kNumTestCases; ++i) {
     out_y[i] = test_cases[i].y;
@@ -349,9 +352,9 @@ void Atan2TestCases(T /*unused*/, D d, size_t& padded,
     out_expected[i] = test_cases[i].expected;
   }
   for (; i < padded; ++i) {
-    out_y[i] = T{0};
-    out_x[i] = T{0};
-    out_expected[i] = T{0};
+    out_y[i] = p0;
+    out_x[i] = p0;
+    out_expected[i] = p0;
   }
 }
 
@@ -364,10 +367,10 @@ struct TestAtan2 {
     AlignedFreeUniquePtr<T[]> in_y, in_x, expected;
     Atan2TestCases(t, d, padded, in_y, in_x, expected);
 
-    const Vec<D> tolerance = Set(d, T(1E-5));
+    const Vec<D> tolerance = Set(d, ConvertScalarTo<T>(1E-5));
 
     for (size_t i = 0; i < padded; ++i) {
-      const T actual = static_cast<T>(atan2(in_y[i], in_x[i]));
+      const T actual = ConvertScalarTo<T>(atan2(in_y[i], in_x[i]));
       // fprintf(stderr, "%zu: table %f atan2 %f\n", i, expected[i], actual);
       HWY_ASSERT_EQ(expected[i], actual);
     }
@@ -393,7 +396,7 @@ struct TestAtan2 {
       if (!AllTrue(d, ok)) {
         const size_t mismatch =
             static_cast<size_t>(FindKnownFirstTrue(d, Not(ok)));
-        fprintf(stderr, "Mismatch for i=%d expected %f actual %f\n",
+        fprintf(stderr, "Mismatch for i=%d expected %E actual %E\n",
                 static_cast<int>(i + mismatch), expected[i + mismatch],
                 ExtractLane(actual, mismatch));
         HWY_ASSERT(0);
@@ -408,6 +411,220 @@ HWY_NOINLINE void TestAllAtan2() {
   ForFloat3264Types(ForPartialVectors<TestAtan2>());
 }
 
+template <typename T, class D>
+void HypotTestCases(T /*unused*/, D d, size_t& padded,
+                    AlignedFreeUniquePtr<T[]>& out_a,
+                    AlignedFreeUniquePtr<T[]>& out_b,
+                    AlignedFreeUniquePtr<T[]>& out_expected) {
+  using TU = MakeUnsigned<T>;
+
+  struct AB {
+    T a;
+    T b;
+  };
+
+  constexpr int kNumOfMantBits = MantissaBits<T>();
+  static_assert(kNumOfMantBits > 0, "kNumOfMantBits > 0 must be true");
+
+  // Ensures inputs are not constexpr.
+  const TU u1 = static_cast<TU>(hwy::Unpredictable1());
+  const double k1 = static_cast<double>(u1);
+
+  const T pos = ConvertScalarTo<T>(1E5 * k1);
+  const T neg = ConvertScalarTo<T>(-1E7 * k1);
+  const T p0 = ConvertScalarTo<T>(k1 - 1.0);
+  // -0 is not enough to get an actual negative zero.
+  const T n0 = ScalarCopySign<T>(p0, neg);
+  const T p1 = ConvertScalarTo<T>(k1);
+  const T n1 = ConvertScalarTo<T>(-k1);
+  const T p2 = ConvertScalarTo<T>(2 * k1);
+  const T n2 = ConvertScalarTo<T>(-2 * k1);
+  const T inf = BitCastScalar<T>(ExponentMask<T>() * u1);
+  const T neg_inf = ScalarCopySign(inf, n1);
+  const T nan = BitCastScalar<T>(
+      static_cast<TU>(ExponentMask<T>() | (u1 << (kNumOfMantBits - 1))));
+
+  const double max_as_f64 = ConvertScalarTo<double>(HighestValue<T>()) * k1;
+  const T max = ConvertScalarTo<T>(max_as_f64);
+
+  const T huge = ConvertScalarTo<T>(max_as_f64 * 0.25);
+  const T neg_huge = ScalarCopySign(huge, n1);
+
+  const T huge2 = ConvertScalarTo<T>(max_as_f64 * 0.039415044328304796);
+
+  const T large = ConvertScalarTo<T>(3.512227595593985E18 * k1);
+  const T neg_large = ScalarCopySign(large, n1);
+  const T large2 = ConvertScalarTo<T>(2.1190576943127544E16 * k1);
+
+  const T small = ConvertScalarTo<T>(1.067033284841808E-11 * k1);
+  const T neg_small = ScalarCopySign(small, n1);
+  const T small2 = ConvertScalarTo<T>(1.9401409532292856E-12 * k1);
+
+  const T tiny = BitCastScalar<T>(static_cast<TU>(u1 << kNumOfMantBits));
+  const T neg_tiny = ScalarCopySign(tiny, n1);
+
+  const T tiny2 =
+      ConvertScalarTo<T>(78.68466968859765 * ConvertScalarTo<double>(tiny));
+
+  const AB test_cases[] = {{p0, p0},          {p0, n0},
+                           {n0, n0},          {p1, p1},
+                           {p1, n1},          {n1, n1},
+                           {p2, p2},          {p2, n2},
+                           {p2, pos},         {p2, neg},
+                           {n2, pos},         {n2, neg},
+                           {n2, n2},          {p0, tiny},
+                           {p0, neg_tiny},    {n0, tiny},
+                           {n0, neg_tiny},    {p1, tiny},
+                           {p1, neg_tiny},    {n1, tiny},
+                           {n1, neg_tiny},    {tiny, p0},
+                           {tiny2, p0},       {tiny, tiny2},
+                           {neg_tiny, tiny2}, {huge, huge2},
+                           {neg_huge, huge2}, {huge, p0},
+                           {huge, tiny},      {huge2, tiny2},
+                           {large, p0},       {large, large2},
+                           {neg_large, p0},   {neg_large, large2},
+                           {small, p0},       {small, small2},
+                           {neg_small, p0},   {neg_small, small2},
+                           {max, p0},         {max, huge},
+                           {max, max},        {p0, inf},
+                           {n0, inf},         {p1, inf},
+                           {n1, inf},         {p2, inf},
+                           {n2, inf},         {p0, neg_inf},
+                           {n0, neg_inf},     {p1, neg_inf},
+                           {n1, neg_inf},     {p2, neg_inf},
+                           {n2, neg_inf},     {p0, nan},
+                           {n0, nan},         {p1, nan},
+                           {n1, nan},         {p2, nan},
+                           {n2, nan},         {huge, inf},
+                           {inf, nan},        {neg_inf, nan},
+                           {nan, nan}};
+
+  const size_t kNumTestCases = sizeof(test_cases) / sizeof(test_cases[0]);
+  const size_t N = Lanes(d);
+  padded = RoundUpTo(kNumTestCases, N);  // allow loading whole vectors
+  out_a = AllocateAligned<T>(padded);
+  out_b = AllocateAligned<T>(padded);
+  out_expected = AllocateAligned<T>(padded);
+  HWY_ASSERT(out_a && out_b && out_expected);
+
+  size_t i = 0;
+  for (; i < kNumTestCases; ++i) {
+    const T a =
+        test_cases[i].a * hwy::ConvertScalarTo<T>(hwy::Unpredictable1());
+    const T b = test_cases[i].b;
+
+#if HWY_TARGET <= HWY_NEON_WITHOUT_AES && HWY_ARCH_ARM_V7
+    // Ignore test cases that have infinite or NaN inputs on Armv7 NEON
+    if (!ScalarIsFinite(a) || !ScalarIsFinite(b)) {
+      out_a[i] = p0;
+      out_b[i] = p0;
+      out_expected[i] = p0;
+      continue;
+    }
+#endif
+
+    out_a[i] = a;
+    out_b[i] = b;
+
+    if (ScalarIsInf(a) || ScalarIsInf(b)) {
+      out_expected[i] = inf;
+    } else if (ScalarIsNaN(a) || ScalarIsNaN(b)) {
+      out_expected[i] = nan;
+    } else {
+      out_expected[i] = std::hypot(a, b);
+    }
+  }
+  for (; i < padded; ++i) {
+    out_a[i] = p0;
+    out_b[i] = p0;
+    out_expected[i] = p0;
+  }
+}
+
+struct TestHypot {
+  template <typename T, class D>
+  HWY_NOINLINE void operator()(T t, D d) {
+    if (HWY_MATH_TEST_EXCESS_PRECISION) {
+      return;
+    }
+
+    const size_t N = Lanes(d);
+
+    constexpr uint64_t kMaxErrorUlp = 4;
+
+    size_t padded;
+    AlignedFreeUniquePtr<T[]> in_a, in_b, expected;
+    HypotTestCases(t, d, padded, in_a, in_b, expected);
+
+    auto actual1_lanes = AllocateAligned<T>(N);
+    auto actual2_lanes = AllocateAligned<T>(N);
+    HWY_ASSERT(actual1_lanes && actual2_lanes);
+
+    uint64_t max_ulp = 0;
+    for (size_t i = 0; i < padded; i += N) {
+      const auto a = Load(d, in_a.get() + i);
+      const auto b = Load(d, in_b.get() + i);
+
+#if HWY_ARCH_ARM_A64
+      // TODO(b/287462770): inline to work around incorrect SVE codegen
+      const auto actual1 = Hypot(d, a, b);
+      const auto actual2 = Hypot(d, b, a);
+#else
+      const auto actual1 = CallHypot(d, a, b);
+      const auto actual2 = CallHypot(d, b, a);
+#endif
+
+      Store(actual1, d, actual1_lanes.get());
+      Store(actual2, d, actual2_lanes.get());
+
+      for (size_t j = 0; j < N; j++) {
+        const T val_a = in_a[i + j];
+        const T val_b = in_b[i + j];
+        const T expected_val = expected[i + j];
+        const T actual1_val = actual1_lanes[j];
+        const T actual2_val = actual2_lanes[j];
+
+        const auto ulp1 =
+            hwy::detail::ComputeUlpDelta(actual1_val, expected_val);
+        if (ulp1 > kMaxErrorUlp) {
+          fprintf(stderr,
+                  "%s: Hypot(%e, %e) lane %d expected %E actual %E ulp %g max "
+                  "ulp %u\n",
+                  hwy::TypeName(T(), Lanes(d)).c_str(), val_a, val_b,
+                  static_cast<int>(j), expected_val, actual1_val,
+                  static_cast<double>(ulp1),
+                  static_cast<uint32_t>(kMaxErrorUlp));
+        }
+
+        const auto ulp2 =
+            hwy::detail::ComputeUlpDelta(actual2_val, expected_val);
+        if (ulp2 > kMaxErrorUlp) {
+          fprintf(stderr,
+                  "%s: Hypot(%e, %e) expected %E actual %E ulp %g max ulp %u\n",
+                  hwy::TypeName(T(), Lanes(d)).c_str(), val_b, val_a,
+                  expected_val, actual2_val, static_cast<double>(ulp2),
+                  static_cast<uint32_t>(kMaxErrorUlp));
+        }
+
+        max_ulp = HWY_MAX(max_ulp, HWY_MAX(ulp1, ulp2));
+      }
+    }
+
+    if (max_ulp != 0) {
+      fprintf(stderr, "%s: Hypot max_ulp %g\n",
+              hwy::TypeName(T(), Lanes(d)).c_str(),
+              static_cast<double>(max_ulp));
+      HWY_ASSERT(max_ulp <= kMaxErrorUlp);
+    }
+  }
+};
+
+HWY_NOINLINE void TestAllHypot() {
+  if (HWY_MATH_TEST_EXCESS_PRECISION) return;
+
+  ForFloat3264Types(ForPartialVectors<TestHypot>());
+}
+
 // NOLINTNEXTLINE(google-readability-namespace-comments)
 }  // namespace HWY_NAMESPACE
 }  // namespace hwy
@@ -425,6 +642,7 @@ HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllAtan);
 HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllAtanh);
 HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllCos);
 HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllExp);
+HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllExp2);
 HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllExpm1);
 HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllLog);
 HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllLog10);
@@ -436,6 +654,8 @@ HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllTanh);
 HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllAtan2);
 HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllSinCosSin);
 HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllSinCosCos);
+HWY_EXPORT_AND_TEST_P(HwyMathTest, TestAllHypot);
+HWY_AFTER_TEST();
 }  // namespace hwy
 
 #endif
