@@ -17,6 +17,7 @@
 
 #include "hwy/examples/skeleton.h"
 
+#include <stdint.h>
 #include <stdio.h>
 
 #undef HWY_TARGET_INCLUDE
@@ -88,18 +89,39 @@ struct TestSumMulAdd {
     auto mul = hwy::AllocateAligned<T>(count);
     auto x = hwy::AllocateAligned<T>(count);
     auto add = hwy::AllocateAligned<T>(count);
+    HWY_ASSERT(mul && x && add);
+
     for (size_t i = 0; i < count; ++i) {
-      mul[i] = static_cast<T>(Random32(&rng) & 0xF);
-      x[i] = static_cast<T>(Random32(&rng) & 0xFF);
-      add[i] = static_cast<T>(Random32(&rng) & 0xFF);
+      mul[i] = hwy::ConvertScalarTo<T>(Random32(&rng) & 0xF);
+      x[i] = hwy::ConvertScalarTo<T>(Random32(&rng) & 0xFF);
+      add[i] = hwy::ConvertScalarTo<T>(Random32(&rng) & 0xFF);
     }
     double expected_sum = 0.0;
     for (size_t i = 0; i < count; ++i) {
-      expected_sum += mul[i] * x[i] + add[i];
+      expected_sum += hwy::ConvertScalarTo<double>(mul[i]) *
+                          hwy::ConvertScalarTo<double>(x[i]) +
+                      hwy::ConvertScalarTo<double>(add[i]);
     }
 
     MulAddLoop(d, mul.get(), add.get(), count, x.get());
+    double vector_sum = 0.0;
+    for (size_t i = 0; i < count; ++i) {
+      vector_sum += x[i];
+    }
+
+    if (hwy::IsSame<T, hwy::float16_t>()) {
+      // The expected value for float16 will vary based on the underlying
+      // implementation (compiler emulation, ARM ACLE __fp16 vs _Float16, etc).
+      // In some cases the scalar and vector paths will have different results;
+      // we check them against known values where possible, else we ignore them.
+#if HWY_COMPILER_CLANG && HWY_NEON_HAVE_F16C
+      HWY_ASSERT_EQ(4344240.0, expected_sum);  // Full-width float
+      HWY_ASSERT_EQ(4344235.0, vector_sum);    // __fp16
+#endif
+      return;
+    }
     HWY_ASSERT_EQ(4344240.0, expected_sum);
+    HWY_ASSERT_EQ(expected_sum, vector_sum);
   }
 };
 
@@ -113,11 +135,10 @@ HWY_NOINLINE void TestAllSumMulAdd() {
 HWY_AFTER_NAMESPACE();
 
 #if HWY_ONCE
-
 namespace skeleton {
 HWY_BEFORE_TEST(SkeletonTest);
 HWY_EXPORT_AND_TEST_P(SkeletonTest, TestAllFloorLog2);
 HWY_EXPORT_AND_TEST_P(SkeletonTest, TestAllSumMulAdd);
+HWY_AFTER_TEST();
 }  // namespace skeleton
-
 #endif
