@@ -13,8 +13,15 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+// Enable all targets so that calling Have* does not call into a null pointer.
+#ifndef HWY_COMPILE_ALL_ATTAINABLE
+#define HWY_COMPILE_ALL_ATTAINABLE
+#endif
 #include "hwy/per_target.h"
 
+#include <stddef.h>
+#include <stdint.h>
+
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "hwy/per_target.cc"
 #include "hwy/foreach_target.h"  // IWYU pragma: keep
@@ -23,6 +30,7 @@
 HWY_BEFORE_NAMESPACE();
 namespace hwy {
 namespace HWY_NAMESPACE {
+int64_t GetTarget() { return HWY_TARGET; }
 size_t GetVectorBytes() { return Lanes(ScalableTag<uint8_t>()); }
 bool GetHaveFloat16() { return HWY_HAVE_FLOAT16 != 0; }
 bool GetHaveFloat64() { return HWY_HAVE_FLOAT64 != 0; }
@@ -35,11 +43,16 @@ HWY_AFTER_NAMESPACE();
 #if HWY_ONCE
 namespace hwy {
 namespace {
+HWY_EXPORT(GetTarget);
 HWY_EXPORT(GetVectorBytes);
 HWY_EXPORT(GetHaveFloat16);
 HWY_EXPORT(GetHaveFloat64);
 }  // namespace
 
+HWY_DLLEXPORT int64_t DispatchedTarget() {
+  return HWY_DYNAMIC_DISPATCH(GetTarget)();
+}
+
 HWY_DLLEXPORT size_t VectorBytes() {
   return HWY_DYNAMIC_DISPATCH(GetVectorBytes)();
 }
