@@ -13,9 +13,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <string.h>
-
-#include <algorithm>  // std::fill
+#include <stddef.h>
+#include <stdint.h>
 
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "tests/blockwise_test.cc"
@@ -37,17 +36,17 @@ struct TestBroadcastR {
     auto in_lanes = AllocateAligned<T>(N);
     auto expected = AllocateAligned<T>(N);
     HWY_ASSERT(in_lanes && expected);
-    std::fill(in_lanes.get(), in_lanes.get() + N, T(0));
+    ZeroBytes(in_lanes.get(), N * sizeof(T));
     const size_t blockN = HWY_MIN(N * sizeof(T), 16) / sizeof(T);
     // Need to set within each 128-bit block
     for (size_t block = 0; block < N; block += blockN) {
-      in_lanes[block + kLane] = static_cast<T>(block + 1);
+      in_lanes[block + kLane] = ConvertScalarTo<T>(block + 1);
     }
     PreventElision(in_lanes[0]);  // workaround for f16x1 failure
     const auto in = Load(d, in_lanes.get());
     for (size_t block = 0; block < N; block += blockN) {
       for (size_t i = 0; i < blockN; ++i) {
-        expected[block + i] = T(block + 1);
+        expected[block + i] = ConvertScalarTo<T>(block + 1);
       }
     }
     HWY_ASSERT_VEC_EQ(d, expected.get(), Broadcast<kLane>(in));
@@ -98,9 +97,9 @@ struct TestTableLookupBytes {
     const size_t N8 = Lanes(d8);
 
     auto in_bytes = AllocateAligned<uint8_t>(NT8);
-    auto index_bytes = AllocateAligned<uint8_t>(N8);
+    auto indices = AllocateAligned<T>(N8);
     auto expected = AllocateAligned<T>(N);
-    HWY_ASSERT(in_bytes && index_bytes && expected);
+    HWY_ASSERT(in_bytes && indices && expected);
 
     // Random input bytes
     for (size_t i = 0; i < NT8; ++i) {
@@ -116,12 +115,13 @@ struct TestTableLookupBytes {
         11, 10, 3, 4, 5,  8,  7,  6,  14, 13, 12, 15, 2,  1,  2,  0,
         4,  3,  2, 2, 5,  6,  7,  7,  15, 15, 15, 15, 15, 15, 0,  1};
     const size_t max_index = HWY_MIN(NT8, 16) - 1;
+    uint8_t* index_bytes = reinterpret_cast<uint8_t*>(indices.get());
     for (size_t i = 0; i < N8; ++i) {
       index_bytes[i] = (i < 64) ? index_bytes_source[i] : 0;
       // Avoid asan error for partial vectors.
       index_bytes[i] = static_cast<uint8_t>(HWY_MIN(index_bytes[i], max_index));
     }
-    const auto indices = Load(d, reinterpret_cast<const T*>(index_bytes.get()));
+    const Vec<D> indices_v = Load(d, indices.get());
 
     uint8_t* expected_bytes = reinterpret_cast<uint8_t*>(expected.get());
 
@@ -138,7 +138,7 @@ struct TestTableLookupBytes {
             in_bytes[(block + index) % HWY_MIN(NT8, 256)];
       }
     }
-    HWY_ASSERT_VEC_EQ(d, expected.get(), TableLookupBytes(in, indices));
+    HWY_ASSERT_VEC_EQ(d, expected.get(), TableLookupBytes(in, indices_v));
 
     // Individually test zeroing each byte position.
     for (size_t i = 0; i < N8; ++i) {
@@ -150,9 +150,8 @@ struct TestTableLookupBytes {
       HWY_ASSERT(0x80 <= idx && idx < 256);
       index_bytes[i] = static_cast<uint8_t>(idx);
 
-      const auto indices =
-          Load(d, reinterpret_cast<const T*>(index_bytes.get()));
-      HWY_ASSERT_VEC_EQ(d, expected.get(), TableLookupBytesOr0(in, indices));
+      const Vec<D> indices_v = Load(d, indices.get());
+      HWY_ASSERT_VEC_EQ(d, expected.get(), TableLookupBytesOr0(in, indices_v));
       expected_bytes[i] = prev_expected;
       index_bytes[i] = prev_index;
     }
@@ -182,8 +181,8 @@ struct TestInterleaveLower {
     auto expected = AllocateAligned<T>(N);
     HWY_ASSERT(even_lanes && odd_lanes && expected);
     for (size_t i = 0; i < N; ++i) {
-      even_lanes[i] = static_cast<T>(2 * i + 0);
-      odd_lanes[i] = static_cast<T>(2 * i + 1);
+      even_lanes[i] = ConvertScalarTo<T>(2 * i + 0);
+      odd_lanes[i] = ConvertScalarTo<T>(2 * i + 1);
     }
     const auto even = Load(d, even_lanes.get());
     const auto odd = Load(d, odd_lanes.get());
@@ -192,7 +191,7 @@ struct TestInterleaveLower {
     for (size_t i = 0; i < Lanes(d); ++i) {
       const size_t block = i / blockN;
       const size_t index = (i % blockN) + block * 2 * blockN;
-      expected[i] = static_cast<T>(index & LimitsMax<TU>());
+      expected[i] = ConvertScalarTo<T>(index & LimitsMax<TU>());
     }
     HWY_ASSERT_VEC_EQ(d, expected.get(), InterleaveLower(even, odd));
     HWY_ASSERT_VEC_EQ(d, expected.get(), InterleaveLower(d, even, odd));
@@ -209,8 +208,8 @@ struct TestInterleaveUpper {
     auto expected = AllocateAligned<T>(N);
     HWY_ASSERT(even_lanes && odd_lanes && expected);
     for (size_t i = 0; i < N; ++i) {
-      even_lanes[i] = static_cast<T>(2 * i + 0);
-      odd_lanes[i] = static_cast<T>(2 * i + 1);
+      even_lanes[i] = ConvertScalarTo<T>(2 * i + 0);
+      odd_lanes[i] = ConvertScalarTo<T>(2 * i + 1);
     }
     const auto even = Load(d, even_lanes.get());
     const auto odd = Load(d, odd_lanes.get());
@@ -218,16 +217,66 @@ struct TestInterleaveUpper {
     const size_t blockN = HWY_MIN(16 / sizeof(T), N);
     for (size_t i = 0; i < Lanes(d); ++i) {
       const size_t block = i / blockN;
-      expected[i] = T((i % blockN) + block * 2 * blockN + blockN);
+      expected[i] =
+          ConvertScalarTo<T>((i % blockN) + block * 2 * blockN + blockN);
     }
     HWY_ASSERT_VEC_EQ(d, expected.get(), InterleaveUpper(d, even, odd));
   }
 };
 
+struct TestInterleaveEven {
+  template <class T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const size_t N = Lanes(d);
+    auto even_lanes = AllocateAligned<T>(N);
+    auto odd_lanes = AllocateAligned<T>(N);
+    auto expected = AllocateAligned<T>(N);
+    HWY_ASSERT(even_lanes && odd_lanes && expected);
+    for (size_t i = 0; i < N; ++i) {
+      even_lanes[i] = ConvertScalarTo<T>(2 * i + 0);
+      odd_lanes[i] = ConvertScalarTo<T>(2 * i + 1);
+    }
+    const auto even = Load(d, even_lanes.get());
+    const auto odd = Load(d, odd_lanes.get());
+
+    for (size_t i = 0; i < N; ++i) {
+      expected[i] = ConvertScalarTo<T>(2 * i - (i & 1));
+    }
+
+    HWY_ASSERT_VEC_EQ(d, expected.get(), InterleaveEven(even, odd));
+    HWY_ASSERT_VEC_EQ(d, expected.get(), InterleaveEven(d, even, odd));
+  }
+};
+
+struct TestInterleaveOdd {
+  template <class T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const size_t N = Lanes(d);
+    auto even_lanes = AllocateAligned<T>(N);
+    auto odd_lanes = AllocateAligned<T>(N);
+    auto expected = AllocateAligned<T>(N);
+    HWY_ASSERT(even_lanes && odd_lanes && expected);
+    for (size_t i = 0; i < N; ++i) {
+      even_lanes[i] = ConvertScalarTo<T>(2 * i + 0);
+      odd_lanes[i] = ConvertScalarTo<T>(2 * i + 1);
+    }
+    const auto even = Load(d, even_lanes.get());
+    const auto odd = Load(d, odd_lanes.get());
+
+    for (size_t i = 0; i < N; ++i) {
+      expected[i] = ConvertScalarTo<T>((2 * i) - (i & 1) + 2);
+    }
+
+    HWY_ASSERT_VEC_EQ(d, expected.get(), InterleaveOdd(d, even, odd));
+  }
+};
+
 HWY_NOINLINE void TestAllInterleave() {
   // Not DemoteVectors because this cannot be supported by HWY_SCALAR.
   ForAllTypes(ForShrinkableVectors<TestInterleaveLower>());
   ForAllTypes(ForShrinkableVectors<TestInterleaveUpper>());
+  ForAllTypes(ForShrinkableVectors<TestInterleaveEven>());
+  ForAllTypes(ForShrinkableVectors<TestInterleaveOdd>());
 }
 
 struct TestZipLower {
@@ -244,8 +293,8 @@ struct TestZipLower {
     HWY_ASSERT(even_lanes && odd_lanes && zip_lanes);
     const T kMaxT = LimitsMax<T>();
     for (size_t i = 0; i < N; ++i) {
-      even_lanes[i] = static_cast<T>((2 * i + 0) & kMaxT);
-      odd_lanes[i] = static_cast<T>((2 * i + 1) & kMaxT);
+      even_lanes[i] = ConvertScalarTo<T>((2 * i + 0) & kMaxT);
+      odd_lanes[i] = ConvertScalarTo<T>((2 * i + 1) & kMaxT);
     }
     const auto even = Load(d, even_lanes.get());
     const auto odd = Load(d, odd_lanes.get());
@@ -319,8 +368,8 @@ struct TestZipUpper {
     HWY_ASSERT(even_lanes && odd_lanes && zip_lanes);
     const T kMaxT = LimitsMax<T>();
     for (size_t i = 0; i < N; ++i) {
-      even_lanes[i] = static_cast<T>((2 * i + 0) & kMaxT);
-      odd_lanes[i] = static_cast<T>((2 * i + 1) & kMaxT);
+      even_lanes[i] = ConvertScalarTo<T>((2 * i + 0) & kMaxT);
+      odd_lanes[i] = ConvertScalarTo<T>((2 * i + 1) & kMaxT);
     }
     const auto even = Load(d, even_lanes.get());
     const auto odd = Load(d, odd_lanes.get());
@@ -389,10 +438,10 @@ class TestSpecialShuffle32 {
     auto expected = AllocateAligned<T>(N);
     HWY_ASSERT(expected);
     for (size_t block = 0; block < N; block += kBlockN) {
-      expected[block + 3] = static_cast<T>(block + i3);
-      expected[block + 2] = static_cast<T>(block + i2);
-      expected[block + 1] = static_cast<T>(block + i1);
-      expected[block + 0] = static_cast<T>(block + i0);
+      expected[block + 3] = ConvertScalarTo<T>(block + i3);
+      expected[block + 2] = ConvertScalarTo<T>(block + i2);
+      expected[block + 1] = ConvertScalarTo<T>(block + i1);
+      expected[block + 0] = ConvertScalarTo<T>(block + i0);
     }
     AssertVecEqual(d, expected.get(), actual, filename, line);
   }
@@ -420,8 +469,8 @@ class TestSpecialShuffle64 {
     auto expected = AllocateAligned<T>(N);
     HWY_ASSERT(expected);
     for (size_t block = 0; block < N; block += kBlockN) {
-      expected[block + 1] = static_cast<T>(block + i1);
-      expected[block + 0] = static_cast<T>(block + i0);
+      expected[block + 1] = ConvertScalarTo<T>(block + i1);
+      expected[block + 0] = ConvertScalarTo<T>(block + i0);
     }
     AssertVecEqual(d, expected.get(), actual, filename, line);
   }
@@ -463,6 +512,7 @@ HWY_EXPORT_AND_TEST_P(HwyBlockwiseTest, TestAllZipLower);
 HWY_EXPORT_AND_TEST_P(HwyBlockwiseTest, TestAllZipUpper);
 #endif
 HWY_EXPORT_AND_TEST_P(HwyBlockwiseTest, TestAllSpecialShuffles);
+HWY_AFTER_TEST();
 }  // namespace hwy
 
 #endif
