@@ -72,7 +72,7 @@ class TestAES {
     }
 
     for (size_t i = 0; i < 256; i += N) {
-      const auto in = Iota(d, static_cast<T>(i));
+      const auto in = Iota(d, i);
       HWY_ASSERT_VEC_EQ(d, expected.get() + i, detail::SubBytes(in));
     }
   }
@@ -165,7 +165,7 @@ class TestAESInverse {
     }
 
     for (size_t i = 0; i < 256; i += N) {
-      const auto in = Iota(d, static_cast<T>(i));
+      const auto in = Iota(d, i);
       HWY_ASSERT_VEC_EQ(d, expected.get() + i, detail::InvSubBytes(in));
     }
   }
@@ -711,6 +711,7 @@ HWY_BEFORE_TEST(HwyCryptoTest);
 HWY_EXPORT_AND_TEST_P(HwyCryptoTest, TestAllAES);
 HWY_EXPORT_AND_TEST_P(HwyCryptoTest, TestAllAESInverse);
 HWY_EXPORT_AND_TEST_P(HwyCryptoTest, TestAllCLMul);
+HWY_AFTER_TEST();
 }  // namespace hwy
 
 #endif
