@@ -33,18 +33,17 @@ class TestSlideUpLanes {
                                             const size_t N,
                                             const size_t slide_amt) {
     for (size_t i = 0; i < N; i++) {
-      expected[i] =
-          static_cast<TFromD<D>>((i >= slide_amt) ? (i - slide_amt + 1) : 0);
+      expected[i] = ConvertScalarTo<TFromD<D>>(
+          (i >= slide_amt) ? (i - slide_amt + 1) : 0);
     }
 
-    const auto v = Iota(d, TFromD<D>{1});
+    const auto v = Iota(d, 1);
     HWY_ASSERT_VEC_EQ(d, expected, SlideUpLanes(d, v, slide_amt));
     if (slide_amt == 1) {
       HWY_ASSERT_VEC_EQ(d, expected, Slide1Up(d, v));
     }
   }
-#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && \
-    HWY_TARGET != HWY_SVE2_128 && HWY_TARGET != HWY_SVE_256
+#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && !HWY_TARGET_IS_SVE
   template <class D>
   static HWY_NOINLINE void DoTestSlideUpLanesWithConstAmt_0_7(
       D d, TFromD<D>* HWY_RESTRICT expected, const size_t N) {
@@ -144,8 +143,7 @@ class TestSlideUpLanes {
   }
 #endif  // HWY_TARGET <= HWY_AVX3
 #endif  // HWY_TARGET <= HWY_AVX2 || HWY_TARGET == HWY_WASM_EMU256
-#endif  // !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 &&
-        // HWY_TARGET != HWY_SVE2_128 && HWY_TARGET != HWY_SVE_256
+#endif  // !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && !HWY_TARGET_IS_SVE
 
  public:
   template <class T, class D>
@@ -162,8 +160,7 @@ class TestSlideUpLanes {
       DoTestSlideUpLanes(d, expected.get(), N, slide_amt);
     }
 
-#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && \
-    HWY_TARGET != HWY_SVE2_128 && HWY_TARGET != HWY_SVE_256
+#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && !HWY_TARGET_IS_SVE
     DoTestSlideUpLanesWithConstAmt_0_7(d, expected.get(), N);
     DoTestSlideUpLanesWithConstAmt_8_15(d, expected.get(), N);
 #if HWY_TARGET <= HWY_AVX2 || HWY_TARGET == HWY_WASM_EMU256
@@ -172,8 +169,7 @@ class TestSlideUpLanes {
     DoTestSlideUpLanesWithConstAmt_32_63(d, expected.get(), N);
 #endif  // HWY_TARGET <= HWY_AVX3
 #endif  // HWY_TARGET <= HWY_AVX2 || HWY_TARGET == HWY_WASM_EMU256
-#endif  // !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 &&
-        // HWY_TARGET != HWY_SVE2_128 && HWY_TARGET != HWY_SVE_256
+#endif  // !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && !HWY_TARGET_IS_SVE
   }
 };
 
@@ -181,8 +177,7 @@ HWY_NOINLINE void TestAllSlideUpLanes() {
   ForAllTypes(ForPartialVectors<TestSlideUpLanes>());
 }
 
-#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && \
-    HWY_TARGET != HWY_SVE2_128 && HWY_TARGET != HWY_SVE_256
+#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && !HWY_TARGET_IS_SVE
 // DoTestSlideDownLanes needs to be inlined on targets where
 // DoTestSlideDownLanesWithConstAmt_0_7, DoTestSlideDownLanesWithConstAmt_8_15,
 // DoTestSlideDownLanesWithConstAmt_16_31, and
@@ -216,17 +211,16 @@ class TestSlideDownLanes {
       const size_t slide_amt) {
     for (size_t i = 0; i < N; i++) {
       const size_t src_idx = slide_amt + i;
-      expected[i] = static_cast<TFromD<D>>((src_idx < N) ? src_idx : 0);
+      expected[i] = ConvertScalarTo<TFromD<D>>((src_idx < N) ? src_idx : 0);
     }
 
-    const Vec<D> v = Iota(d, TFromD<D>{0});
+    const Vec<D> v = Iota(d, 0);
     HWY_ASSERT_VEC_EQ(d, expected, SlideDownLanes(d, v, slide_amt));
     if (slide_amt == 1) {
       HWY_ASSERT_VEC_EQ(d, expected, Slide1Down(d, v));
     }
   }
-#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && \
-    HWY_TARGET != HWY_SVE2_128 && HWY_TARGET != HWY_SVE_256
+#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && !HWY_TARGET_IS_SVE
   template <class D>
   static HWY_NOINLINE void DoTestSlideDownLanesWithConstAmt_0_7(
       D d, TFromD<D>* HWY_RESTRICT expected, const size_t N) {
@@ -326,8 +320,7 @@ class TestSlideDownLanes {
   }
 #endif  // HWY_TARGET <= HWY_AVX3
 #endif  // HWY_TARGET <= HWY_AVX2 || HWY_TARGET == HWY_WASM_EMU256
-#endif  // !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 &&
-        // HWY_TARGET != HWY_SVE2_128 && HWY_TARGET != HWY_SVE_256
+#endif  // !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && !HWY_TARGET_IS_SVE
 
  public:
   template <class T, class D>
@@ -344,8 +337,7 @@ class TestSlideDownLanes {
       DoTestSlideDownLanes(d, expected.get(), N, slide_amt);
     }
 
-#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && \
-    HWY_TARGET != HWY_SVE2_128 && HWY_TARGET != HWY_SVE_256
+#if !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && !HWY_TARGET_IS_SVE
     DoTestSlideDownLanesWithConstAmt_0_7(d, expected.get(), N);
     DoTestSlideDownLanesWithConstAmt_8_15(d, expected.get(), N);
 #if HWY_TARGET <= HWY_AVX2 || HWY_TARGET == HWY_WASM_EMU256
@@ -354,8 +346,7 @@ class TestSlideDownLanes {
     DoTestSlideDownLanesWithConstAmt_32_63(d, expected.get(), N);
 #endif  // HWY_TARGET <= HWY_AVX3
 #endif  // HWY_TARGET <= HWY_AVX2 || HWY_TARGET == HWY_WASM_EMU256
-#endif  // !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 &&
-        // HWY_TARGET != HWY_SVE2_128 && HWY_TARGET != HWY_SVE_256
+#endif  // !HWY_HAVE_SCALABLE && HWY_TARGET < HWY_EMU128 && !HWY_TARGET_IS_SVE
   }
 };
 
@@ -368,8 +359,8 @@ HWY_NOINLINE void TestAllSlideDownLanes() {
 struct TestSlide1 {
   template <class T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
-    const auto iota0 = Iota(d, T{0});
-    const auto iota1 = Iota(d, T{1});
+    const auto iota0 = Iota(d, 0);
+    const auto iota1 = Iota(d, 1);
 
     const auto expected_slide_down_result =
         IfThenElseZero(FirstN(d, Lanes(d) - 1), iota1);
@@ -392,8 +383,8 @@ class TestSlideBlocks {
     constexpr size_t kLanesToSlide =
         static_cast<size_t>(kBlocks) * kLanesPerBlock;
 
-    const auto iota_0 = Iota(d, T{0});
-    const auto iota_k = Iota(d, static_cast<T>(kLanesToSlide));
+    const auto iota_0 = Iota(d, 0);
+    const auto iota_k = Iota(d, kLanesToSlide);
 
     const auto first_k_lanes_mask = FirstN(d, kLanesToSlide);
     const auto expected_slide_up_result =
@@ -405,7 +396,7 @@ class TestSlideBlocks {
     using TU = TFromD<decltype(du)>;
     const auto slide_down_result_mask = BitCast(
         d, Reverse(du, IfThenZeroElse(RebindMask(du, first_k_lanes_mask),
-                                      Set(du, static_cast<TU>(-1)))));
+                                      Set(du, hwy::LimitsMax<TU>()))));
 
     const auto expected_slide_down_result = And(slide_down_result_mask, iota_k);
     HWY_ASSERT_VEC_EQ(d, expected_slide_down_result,
@@ -466,6 +457,7 @@ HWY_EXPORT_AND_TEST_P(HwySlideUpDownTest, TestAllSlideUpLanes);
 HWY_EXPORT_AND_TEST_P(HwySlideUpDownTest, TestAllSlideDownLanes);
 HWY_EXPORT_AND_TEST_P(HwySlideUpDownTest, TestAllSlide1);
 HWY_EXPORT_AND_TEST_P(HwySlideUpDownTest, TestAllSlideBlocks);
+HWY_AFTER_TEST();
 }  // namespace hwy
 
 #endif
